/*
   This Tree<T> is generic in the modern Java sense.

   But there is no well defined "empty tree". We
   are using null to denote an empty tree, but a
   null value could also be a programming error.
*/
public class Tree<T>
{
   public T data;
   public Tree<T> left;
   public Tree<T> right;

   public Tree(T data) {
      this(data, null, null);
   }

   public Tree(T data, Tree<T> left, Tree<T> right) {
      this.data = data;
      this.left = left;
      this.right = right;
   }

   public int depth() {
      int result = 1;
      if (null != left) {
         result = left.depth();  // recursion
      }
      if (null != right) {
         result = Math.max(result, right.depth()); // recursion
      }
      return result;
   }

   public int size() {
      int result = 1;
      if (null != left) {
         result += left.size(); // recursion
      }
      if (null != right) {
         result += right.size(); // recursion
      }
      return result;
   }

   public String toString() {
      String result = "(" + data;
      result += ", ";
      result += left;  // recursion
      result += ", ";
      result += right; // recursion
      result += ")";
      return result;
   }
}
