/*
   This Tree<T> is generic in the modern Java sense.

   This tree enforces a rule that the data field
   cannot be null (by making data private).

   Since data is private, we need an accessor method.

   But we are still using null to denote "empty tree".
*/
public class Tree<T>
{
   private T data;
   public  Tree<T> left;
   public  Tree<T> right;

   public Tree(T data) {
      this(data, null, null);
   }

   public Tree(T data, Tree<T> left, Tree<T> right) {
      if (null == data)
         throw new IllegalArgumentException();

      this.data = data;
      this.left = left;
      this.right = right;
   }

   public T root() {
      return data;
   }

   public int depth() {
      int result = 1;
      if (null != left) {
         result = left.depth();  // recursion
      }
      if (null != right) {
         result = Math.max(result, right.depth()); // recursion
      }
      return result;
   }

   public int size() {
      int result = 1;
      if (null != left) {
         result += left.size();  // recursion
      }
      if (null != right) {
         result += right.size(); // recursion
      }
      return result;
   }

   public String toString() {
      String result = "(" + data;
      result += ", ";
      result += left;  // recursion
      result += ", ";
      result += right; // recursion
      result += ")";
      return result;
   }
}
