/*
   This Tree<T> is generic in the modern Java sense.

   This tree enforces a rule that the data field
   cannot be null (by making data private).

   These trees are immutable (since the left and
   right sub-trees are private). There is no way
   to change a Tree once it has been constructed.

   Since data, left, and right are all private, we
   need an accessor method for each one.

   Using the Tree constructors we can build arbitrary
   trees but we must build them "from the bottom up".

   But we are still using null to denote "empty tree".
*/
public class Tree<T>
{
   private T data;
   private Tree<T> left;
   private Tree<T> right;

   public Tree(T data) {
      this(data, null, null);
   }

   public Tree(T data, Tree<T> left, Tree<T> right) {
      if (null == data)
         throw new IllegalArgumentException();

      this.data = data;
      this.left = left;
      this.right = right;
   }

   public T root() {
      return data;
   }

   public Tree<T> left() {
      return left;
   }

   public Tree<T> right() {
      return right;
   }

   public int depth() {
      int result = 1;
      if (null != left) {
         result = left.depth();
      }
      if (null != right) {
         result = Math.max(result, right.depth());
      }
      return result;
   }

   public int size() {
      int result = 1;
      if (null != left) {
         result += left.size();
      }
      if (null != right) {
         result += right.size();
      }
      return result;
   }

   public String toString() {
      String result = "(" + data;
      result += ", ";
      result += left;  // recursion
      result += ", ";
      result += right; // recursion
      result += ")";
      return result;
   }
}
